/*#- macro entry_name(namespace, struct) -#*/
/*{namespace.name}*/_/*{struct.name}*/
/*#- endmacro #*/

/*#- macro cdecls(struct) -#*/
/*{struct.non_reserved_members
	| map(attribute="const_cdecl")
	| join(",\n\t\t")}*/
/*#- endmacro -#*/

#pragma once

/* Generated by xpdt
 * xpdt is written by Gianni Tedesco
 * https://github.com/giannitedesco/xpdt
 */

/*# for hdr in headers #*/
#include /*{hdr.path_spec}*/
/*# endfor #*/
/*#- for struct in namespace #*/

void /*{entry_name(namespace, struct)}*/(/*{cdecls(struct)}*/);
/*#- endfor #*/
